import 'dart:convert';

import 'package:http/http.dart';
import 'package:twilio_conversation_rest/models/utils/exception.dart';
import 'package:twilio_conversation_rest/models/utils/multi_data.dart';
import 'package:twilio_conversation_rest/models/twi_conv.dart';
import 'package:twilio_conversation_rest/models/twi_mesg.dart';
import 'package:twilio_conversation_rest/models/twi_part.dart';

abstract class TwiConversationsApis<T>{
  final String accSid;
  final String authToken;
  final String? conversationServiceSID;
  String get apiPath;
  String get baseConvPath=>"/v1/${conversationServiceSID==null?"":
  "Services/$conversationServiceSID/"}Conversations";

  Uri tUri({String? id})=>Uri(scheme: "https",userInfo: "$accSid:$authToken",
    host: "conversations.twilio.com",path: "$baseConvPath$apiPath${id==null?"":"/$id"}");
  TwiConversationsApis(this.accSid, this.authToken,{this.conversationServiceSID});

  Future<T> create(Map<String,dynamic>body)async{
    final r=await post(tUri(),body: body,headers: {
      "Content-Type": "application/x-www-form-urlencoded",},
      encoding: Encoding.getByName('utf-8'),);
    if(r.statusCode>=400) throw TwiException.fromResponse(r.body);
    return returnT(json.decode(utf8.decode(r.bodyBytes)));
  }

  Future<T> fetch(String id)async{
    final r=await get(tUri(id:id));
    if(r.statusCode>=400) throw TwiException.fromResponse(r.body);
    return returnT(json.decode(utf8.decode(r.bodyBytes)));
  }

  Future<MultiData<T>> readMulti(Map<String,dynamic>qPara)async{
    final r=await get(tUri().replace(queryParameters: qPara));
    if(r.statusCode>=400) throw TwiException.fromResponse(r.body);
    return MultiData<T>.fromMap(json.decode(utf8.decode(r.bodyBytes)));
  }

  Future<T> update(String sid,Map<String,dynamic>body)async{
    final r=await post(tUri(id: sid),body: body,headers: {
      "Content-Type": "application/x-www-form-urlencoded",},
      encoding: Encoding.getByName('utf-8'),);
    if(r.statusCode>=400) throw TwiException.fromResponse(r.body);
    return returnT(json.decode(utf8.decode(r.bodyBytes)));
  }

  Future<void> delet(String id)async{
    final r=await delete(tUri(id: id));
    if(r.statusCode>=400) throw TwiException.fromResponse(r.body);
  }

  T returnT(Map<String,dynamic>d){
    if(T == TwiConv)return TwiConv.fromJson(d) as T;
    if(T == TwiPart)return TwiPart.fromJson(d) as T;
    if(T == TwiMesg)return TwiMesg.fromJson(d) as T;
    throw Exception("TwiConversationsApi Error: unknown type from TwiConv,TwiPart,or TwiMesg");
  }
}